@@ -1,18 +1,15 @@
-import pyranges as pr
+from collections import defaultdict
+
 import pandas as pd
 
-from collections import defaultdict
+import pyranges as pr
 
 
 def concat(pyranges, strand=None):
-
     if not pyranges:
         return None
 
-    # from pydbg import dbg
     pyranges = [pr for pr in pyranges if not pr.empty]
-    # dbg(pyranges)
-    # dbg([p.df.dtypes for p in pyranges])
     grs_per_chromosome = defaultdict(list)
 
     strand_info = [gr.stranded for gr in pyranges]
@@ -21,9 +18,7 @@ def concat(pyranges, strand=None):
         strand = all(strand_info)
 
     if strand:
-        assert all([
-            gr.stranded for gr in pyranges
-        ]), "Cannot do stranded concat, not all pyranges contain strand info."
+        assert all([gr.stranded for gr in pyranges]), "Cannot do stranded concat, not all pyranges contain strand info."
 
         for gr in pyranges:
             for k, df in gr.dfs.items():
@@ -40,16 +35,16 @@ def concat(pyranges, strand=None):
     for k, v in grs_per_chromosome.items():
         new_pyrange[k] = pd.concat(v, sort=False)
 
-    res = pr.multithreaded.process_results(new_pyrange.values(),
-                                           new_pyrange.keys())
+    res = pr.multithreaded.process_results(new_pyrange.values(), new_pyrange.keys())
 
     if any(strand_info) and not all(strand_info):
         new_res = {}
         for k, v in res.items():
+            v.loc[:, "Strand"] = v.Strand.cat.add_categories(["."])
             new_res[k] = v.assign(Strand=v.Strand.fillna("."))
         res = pr.PyRanges(new_res)
         res.Strand = res.Strand
     else:
         res = pr.PyRanges(res)
 
-    return  res
+    return res
