@@ -3,17 +3,13 @@ from __future__ import print_function
 import sys
 
 import pandas as pd
-
-from pyranges.pyranges import PyRanges
-
-from io import StringIO
+from natsort import natsorted  # type: ignore
 
 import pyranges as pr
-from pyranges.version import __version__
+from pyranges.pyranges_main import PyRanges
 
 
 def read_bed(f, as_df=False, nrows=None):
-
     """Return bed file as PyRanges.
 
     This is a reader for files that follow the bed format. They can have from
@@ -49,7 +45,7 @@ def read_bed(f, as_df=False, nrows=None):
     >>> pr.read_bed(path, nrows=5)
     +--------------+-----------+-----------+------------+-----------+--------------+
     | Chromosome   |     Start |       End | Name       |     Score | Strand       |
-    | (category)   |   (int32) |   (int32) | (object)   |   (int64) | (category)   |
+    | (category)   |   (int64) |   (int64) | (object)   |   (int64) | (category)   |
     |--------------+-----------+-----------+------------+-----------+--------------|
     | chr1         |      9939 |     10138 | H3K27me3   |         7 | +            |
     | chr1         |      9953 |     10152 | H3K27me3   |         5 | +            |
@@ -67,13 +63,16 @@ def read_bed(f, as_df=False, nrows=None):
     2       chr1   9951  10150  H3K27me3      8      -
     3       chr1   9953  10152  H3K27me3      5      +
     4       chr1   9978  10177  H3K27me3      7      -
+
     """
 
-    columns = "Chromosome Start End Name Score Strand ThickStart ThickEnd ItemRGB BlockCount BlockSizes BlockStarts".split(
+    columns = (
+        "Chromosome Start End Name Score Strand ThickStart ThickEnd ItemRGB BlockCount BlockSizes BlockStarts".split()
     )
 
     if f.endswith(".gz"):
         import gzip
+
         first_start = gzip.open(f).readline().split()[1]
     else:
         first_start = open(f).readline().split()[1]
@@ -87,15 +86,13 @@ def read_bed(f, as_df=False, nrows=None):
 
     df = pd.read_csv(
         f,
-        dtype={
-            "Chromosome": "category",
-            "Strand": "category"
-        },
+        dtype={"Chromosome": "category", "Strand": "category"},
         nrows=nrows,
         header=header,
-        sep="\t")
+        sep="\t",
+    )
 
-    df.columns = columns[:df.shape[1]]
+    df.columns = columns[: df.shape[1]]
 
     if not as_df:
         return PyRanges(df)
@@ -104,7 +101,6 @@ def read_bed(f, as_df=False, nrows=None):
 
 
 def read_bam(f, sparse=True, as_df=False, mapq=0, required_flag=0, filter_flag=1540):
-
     """Return bam file as PyRanges.
 
     Parameters
@@ -145,50 +141,47 @@ def read_bam(f, sparse=True, as_df=False, mapq=0, required_flag=0, filter_flag=1
     --------
 
     >>> path = pr.get_example_path("control.bam")
-    >>> pr.read_bam(path)
+    >>> pr.read_bam(path).sort()
     +--------------+-----------+-----------+--------------+------------+
     | Chromosome   | Start     | End       | Strand       | Flag       |
-    | (category)   | (int32)   | (int32)   | (category)   | (uint16)   |
+    | (category)   | (int64)   | (int64)   | (category)   | (uint16)   |
     |--------------+-----------+-----------+--------------+------------|
-    | chr1         | 887771    | 887796    | +            | 16         |
-    | chr1         | 994660    | 994685    | +            | 16         |
-    | chr1         | 1770383   | 1770408   | +            | 16         |
-    | chr1         | 1995141   | 1995166   | +            | 16         |
+    | chr1         | 1041102   | 1041127   | +            | 0          |
+    | chr1         | 2129359   | 2129384   | +            | 0          |
+    | chr1         | 2239108   | 2239133   | +            | 0          |
+    | chr1         | 2318805   | 2318830   | +            | 0          |
     | ...          | ...       | ...       | ...          | ...        |
-    | chrY         | 57402214  | 57402239  | +            | 16         |
-    | chrY         | 10643526  | 10643551  | -            | 0          |
-    | chrY         | 11776321  | 11776346  | -            | 0          |
-    | chrY         | 20557165  | 20557190  | -            | 0          |
+    | chrY         | 10632456  | 10632481  | -            | 16         |
+    | chrY         | 11918814  | 11918839  | -            | 16         |
+    | chrY         | 11936866  | 11936891  | -            | 16         |
+    | chrY         | 57402214  | 57402239  | -            | 16         |
     +--------------+-----------+-----------+--------------+------------+
     Stranded PyRanges object has 10,000 rows and 5 columns from 25 chromosomes.
     For printing, the PyRanges was sorted on Chromosome and Strand.
-
-    >>> pr.read_bam(path, sparse=False, as_df=True)
-         Chromosome    Start      End Strand  Flag  QueryStart  QueryEnd Name Cigar Quality
-    0          chr1   887771   887796      +    16           0        25   U0   25M    None
-    1          chr1   994660   994685      +    16           0        25   U0   25M    None
-    2          chr1  1041102  1041127      -     0           0        25   U0   25M    None
-    3          chr1  1770383  1770408      +    16           0        25   U0   25M    None
-    4          chr1  1995141  1995166      +    16           0        25   U0   25M    None
-    ...         ...      ...      ...    ...   ...         ...       ...  ...   ...     ...
-    9995       chrM     3654     3679      -     0           0        25   U0   25M    None
-    9996       chrM     3900     3925      +    16           0        25   U0   25M    None
-    9997       chrM    13006    13031      +    16           0        25   U0   25M    None
-    9998       chrM    14257    14282      -     0           0        25   U0   25M    None
-    9999       chrM    14257    14282      -     0           0        25   U0   25M    None
-    <BLANKLINE>
-    [10000 rows x 10 columns]
     """
 
     try:
-        import bamread
-    except ModuleNotFoundError as e:
-        print("bamread must be installed to read bam. Use `conda install -c bioconda bamread` or `pip install bamread` to install it.")
+        import bamread  # type: ignore
+    except ImportError:
+        print(
+            "bamread must be installed to read bam. Use `conda install -c bioconda bamread` or `pip install bamread` to install it."
+        )
         sys.exit(1)
 
-    if bamread.__version__ in ["0.0.1", "0.0.2", "0.0.3", "0.0.4",
-                               "0.0.5", "0.0.6", "0.0.7", "0.0.8", "0.0.9"]:
-        print("bamread not recent enough. Must be 0.0.10 or higher. Use `conda install -c bioconda 'bamread>=0.0.10'` or `pip install bamread>=0.0.10` to install it.")
+    if bamread.__version__ in [
+        "0.0.1",
+        "0.0.2",
+        "0.0.3",
+        "0.0.4",
+        "0.0.5",
+        "0.0.6",
+        "0.0.7",
+        "0.0.8",
+        "0.0.9",
+    ]:
+        print(
+            "bamread not recent enough. Must be 0.0.10 or higher. Use `conda install -c bioconda 'bamread>=0.0.10'` or `pip install bamread>=0.0.10` to install it."
+        )
         sys.exit(1)
 
     if sparse:
@@ -208,19 +201,19 @@ def read_bam(f, sparse=True, as_df=False, mapq=0, required_flag=0, filter_flag=1
 
 
 def _fetch_gene_transcript_exon_id(attribute, annotation=None):
-
-    no_quotes = attribute.str.replace('"', '').str.replace("'", "")
+    no_quotes = attribute.str.replace('"', "").str.replace("'", "")
 
     df = no_quotes.str.extract(
         "gene_id.?(.+?);(?:.*transcript_id.?(.+?);)?(?:.*exon_number.?(.+?);)?(?:.*exon_id.?(.+?);)?",
-        expand=True)  # .iloc[:, [1, 2, 3]]
+        expand=True,
+    )  # .iloc[:, [1, 2, 3]]
 
     df.columns = "gene_id transcript_id exon_number exon_id".split()
 
     if annotation == "ensembl":
         newdf = []
         for c in "gene_id transcript_id exon_id".split():
-            r = df[c].astype(str).str.extract(r'(\d+)').astype(float)
+            r = df[c].astype(str).str.extract(r"(\d+)").astype(float)
             newdf.append(r)
 
         newdf = pd.concat(newdf, axis=1)
@@ -231,9 +224,9 @@ def _fetch_gene_transcript_exon_id(attribute, annotation=None):
 
 
 def skiprows(f):
-
     try:
         import gzip
+
         fh = gzip.open(f)
         for i, l in enumerate(fh):
             if l.decode()[0] != "#":
@@ -249,8 +242,14 @@ def skiprows(f):
     return i
 
 
-def read_gtf(f, full=True, as_df=False, nrows=None, duplicate_attr=False):
-
+def read_gtf(
+    f,
+    full=True,
+    as_df=False,
+    nrows=None,
+    duplicate_attr=False,
+    ignore_bad: bool = False,
+):
     """Read files in the Gene Transfer Format.
 
     Parameters
@@ -275,6 +274,17 @@ def read_gtf(f, full=True, as_df=False, nrows=None, duplicate_attr=False):
 
         Whether to handle (potential) duplicate attributes or just keep last one.
 
+    ignore_bad : bool, default False
+
+        Whether to ignore bad lines or raise an error.
+
+    Note
+    ----
+
+    The GTF format encodes both Start and End as 1-based included.
+    PyRanges (and also the DF returned by this function, if as_df=True), instead
+    encodes intervals as 0-based, Start included and End excluded.
+
     See Also
     --------
 
@@ -288,7 +298,7 @@ def read_gtf(f, full=True, as_df=False, nrows=None, duplicate_attr=False):
 
     >>> # +--------------+------------+--------------+-----------+-----------+------------+--------------+------------+-----------------+----------------+-------+
     >>> # | Chromosome   | Source     | Feature      | Start     | End       | Score      | Strand       | Frame      | gene_id         | gene_version   | +18   |
-    >>> # | (category)   | (object)   | (category)   | (int32)   | (int32)   | (object)   | (category)   | (object)   | (object)        | (object)       | ...   |
+    >>> # | (category)   | (object)   | (category)   | (int64)   | (int64)   | (object)   | (category)   | (object)   | (object)        | (object)       | ...   |
     >>> # |--------------+------------+--------------+-----------+-----------+------------+--------------+------------+-----------------+----------------+-------|
     >>> # | 1            | havana     | gene         | 11868     | 14409     | .          | +            | .          | ENSG00000223972 | 5              | ...   |
     >>> # | 1            | havana     | transcript   | 11868     | 14409     | .          | +            | .          | ENSG00000223972 | 5              | ...   |
@@ -308,40 +318,44 @@ def read_gtf(f, full=True, as_df=False, nrows=None, duplicate_attr=False):
     _skiprows = skiprows(f)
 
     if full:
-        gr = read_gtf_full(f, as_df, nrows, _skiprows, duplicate_attr)
+        gr = read_gtf_full(f, as_df, nrows, _skiprows, duplicate_attr, ignore_bad=ignore_bad)
     else:
         gr = read_gtf_restricted(f, _skiprows, as_df=False, nrows=None)
 
     return gr
 
 
-def read_gtf_full(f, as_df=False, nrows=None, skiprows=0, duplicate_attr=False):
+def read_gtf_full(
+    f,
+    as_df=False,
+    nrows=None,
+    skiprows=0,
+    duplicate_attr=False,
+    ignore_bad: bool = False,
+    chunksize: int = int(1e5),  # for unit-testing purposes
+):
+    dtypes = {"Chromosome": "category", "Feature": "category", "Strand": "category"}
 
-    dtypes = {
-        "Chromosome": "category",
-        "Feature": "category",
-        "Strand": "category"
-    }
-
-    names = "Chromosome Source Feature Start End Score Strand Frame Attribute".split(
-    )
+    names = "Chromosome Source Feature Start End Score Strand Frame Attribute".split()
 
     df_iter = pd.read_csv(
         f,
         sep="\t",
         header=None,
         names=names,
-        dtype=dtypes,
-        chunksize=int(1e5),
+        dtype=dtypes,  # type: ignore
+        chunksize=chunksize,
         skiprows=skiprows,
-        nrows=nrows)
+        nrows=nrows,
+    )
 
     _to_rows = to_rows_keep_duplicates if duplicate_attr else to_rows
 
     dfs = []
     for df in df_iter:
-        extra = _to_rows(df.Attribute)
+        extra = _to_rows(df.Attribute, ignore_bad=ignore_bad)
         df = df.drop("Attribute", axis=1)
+        extra.set_index(df.index, inplace=True)
         ndf = pd.concat([df, extra], axis=1, sort=False)
         dfs.append(ndf)
 
@@ -354,50 +368,60 @@ def read_gtf_full(f, as_df=False, nrows=None, skiprows=0, duplicate_attr=False):
         return df
 
 
-def to_rows(anno):
+def parse_kv_fields(line):
+    # rstrip: allows for GFF not having a last ";", or having final spaces
+    return [kv.replace('""', '"NA"').replace('"', "").split(None, 1) for kv in line.rstrip("; ").split("; ")]
+
+
+def to_rows(anno, ignore_bad: bool = False):
     rowdicts = []
     try:
-        l = anno.head(1)
-        for l in l:
-            l.replace('"', '').replace(";", "").split()
+        line = anno.head(1)
+        for line in line:
+            line.replace('"', "").replace(";", "").split()
     except AttributeError:
-        raise Exception("Invalid attribute string: {l}. If the file is in GFF3 format, use pr.read_gff3 instead.".format(l=l))
+        raise Exception(
+            "Invalid attribute string: {line}. If the file is in GFF3 format, use pr.read_gff3 instead.".format(
+                line=line
+            )
+        )
 
-    for l in anno:
-        rowdicts.append({k: v
-                         for k, v in [kv.replace('""', '"NA"').replace('"', '').split(None, 1)
-                                      for kv in l[:-1].split("; ")]})
+    try:
+        for line in anno:
+            rowdicts.append({k: v for k, v in parse_kv_fields(line)})
+    except ValueError:
+        if not ignore_bad:
+            print(f"The following line is not parseable as gtf:\n{line}\n\nTo ignore bad lines use ignore_bad=True.")
+            raise
 
-    return pd.DataFrame.from_dict(rowdicts).set_index(anno.index)
+    return pd.DataFrame.from_records(rowdicts)
 
 
-def to_rows_keep_duplicates(anno):
+def to_rows_keep_duplicates(anno, ignore_bad: bool = False):
     rowdicts = []
-    for l in anno:
-        rowdict = {}
+    try:
+        for line in anno:
+            rowdict = {}
 
-        # l[:-1] removes final ";" cheaply
-        for k, v in (kv.replace('"', '').split(None, 1) for kv in l[:-1].split("; ")):
+            # rstrip: allows for GFF not having a last ";", or having final spaces
+            for k, v in tuple(parse_kv_fields(line)):
+                if k not in rowdict:
+                    rowdict[k] = v
+                elif k in rowdict and isinstance(rowdict[k], list):
+                    rowdict[k].append(v)
+                else:
+                    rowdict[k] = [rowdict[k], v]
 
-            if k not in rowdict:
-                rowdict[k] = v
-            elif k in rowdict and isinstance(rowdict[k], list):
-                rowdict[k].append(v)
-            else:
-                rowdict[k] = [rowdict[k], v]
+            rowdicts.append({k: ",".join(v) if isinstance(v, list) else v for k, v in rowdict.items()})
+    except ValueError:
+        if not ignore_bad:
+            print(f"The following line is not parseable as gtf:\n\n{line}\n\nTo ignore bad lines use ignore_bad=True.")
+            raise
 
-        rowdicts.append({
-            k: ','.join(v) if isinstance(v, list) else v
-            for k, v in rowdict.items()
-        })
-
-    return pd.DataFrame.from_dict(rowdicts).set_index(anno.index)
+    return pd.DataFrame.from_records(rowdicts)
 
 
-def read_gtf_restricted(f,
-                        skiprows,
-                        as_df=False,
-                        nrows=None):
+def read_gtf_restricted(f, skiprows, as_df=False, nrows=None):
     """seqname - name of the chromosome or scaffold; chromosome names can be given with or without the 'chr' prefix. Important note: the seqname must be one used within Ensembl, i.e. a standard chromosome name or an Ensembl identifier such as a scaffold ID, without any additional content such as species or assembly. See the example GFF output below.
     # source - name of the program that generated this feature, or the data source (database or project name)
     feature - feature type name, e.g. Gene, Variation, Similarity
@@ -406,12 +430,9 @@ def read_gtf_restricted(f,
     score - A floating point value.
     strand - defined as + (forward) or - (reverse).
     # frame - One of '0', '1' or '2'. '0' indicates that the first base of the feature is the first base of a codon, '1' that the second base is the first base of a codon, and so on..
-    attribute - A semicolon-separated list of tag-value pairs, providing additional information about each feature."""
-    dtypes = {
-        "Chromosome": "category",
-        "Feature": "category",
-        "Strand": "category"
-    }
+    attribute - A semicolon-separated list of tag-value pairs, providing additional information about each feature.
+    """
+    dtypes = {"Chromosome": "category", "Feature": "category", "Strand": "category"}
 
     df_iter = pd.read_csv(
         f,
@@ -423,21 +444,22 @@ def read_gtf_restricted(f,
         dtype=dtypes,
         chunksize=int(1e5),
         skiprows=skiprows,
-        nrows=nrows)
+        nrows=nrows,
+    )
 
     dfs = []
     for df in df_iter:
         if sum(df.Score == ".") == len(df):
             cols_to_concat = "Chromosome Start End Strand Feature".split()
         else:
-            cols_to_concat = "Chromosome Start End Strand Feature Score".split(
-            )
+            cols_to_concat = "Chromosome Start End Strand Feature Score".split()
 
         extract = _fetch_gene_transcript_exon_id(df.Attribute)
         extract.columns = "gene_id transcript_id exon_number exon_id".split()
 
         extract.exon_number = extract.exon_number.astype(float)
 
+        extract.set_index(df.index, inplace=True)
         df = pd.concat([df[cols_to_concat], extract], axis=1, sort=False)
 
         dfs.append(df)
@@ -455,15 +477,15 @@ def read_gtf_restricted(f,
 def to_rows_gff3(anno):
     rowdicts = []
 
-    for l in list(anno):
-        l = (it.split("=") for it in l.split(";"))
-        rowdicts.append({k: v for k, v in l})
+    for line in list(anno):
+        # stripping last white char if present
+        lx = (it.split("=") for it in line.rstrip("; ").split(";"))
+        rowdicts.append({k: v for k, v in lx})
 
-    return pd.DataFrame.from_dict(rowdicts).set_index(anno.index)
+    return pd.DataFrame.from_records(rowdicts).set_index(anno.index)
 
 
 def read_gff3(f, full=True, annotation=None, as_df=False, nrows=None):
-
     """Read files in the General Feature Format.
 
     Parameters
@@ -484,6 +506,13 @@ def read_gff3(f, full=True, annotation=None, as_df=False, nrows=None):
 
         Number of rows to read. Default None, i.e. all.
 
+    Notes
+    -----
+
+    The gff3 format encodes both Start and End as 1-based included.
+    PyRanges (and also the DF returned by this function, if as_df=True), instead
+    encodes intervals as 0-based, Start included and End excluded.
+
     See Also
     --------
 
@@ -495,14 +524,9 @@ def read_gff3(f, full=True, annotation=None, as_df=False, nrows=None):
     if not full:
         return read_gtf_restricted(f, _skiprows, as_df=as_df, nrows=nrows)
 
-    dtypes = {
-        "Chromosome": "category",
-        "Feature": "category",
-        "Strand": "category"
-    }
+    dtypes = {"Chromosome": "category", "Feature": "category", "Strand": "category"}
 
-    names = "Chromosome Source Feature Start End Score Strand Frame Attribute".split(
-    )
+    names = "Chromosome Source Feature Start End Score Strand Frame Attribute".split()
 
     df_iter = pd.read_csv(
         f,
@@ -513,12 +537,14 @@ def read_gff3(f, full=True, annotation=None, as_df=False, nrows=None):
         dtype=dtypes,
         chunksize=int(1e5),
         skiprows=_skiprows,
-        nrows=nrows)
+        nrows=nrows,
+    )
 
     dfs = []
     for df in df_iter:
         extra = to_rows_gff3(df.Attribute.astype(str))
         df = df.drop("Attribute", axis=1)
+        extra.set_index(df.index, inplace=True)
         ndf = pd.concat([df, extra], axis=1, sort=False)
         dfs.append(ndf)
 
@@ -533,12 +559,14 @@ def read_gff3(f, full=True, annotation=None, as_df=False, nrows=None):
 
 
 def read_bigwig(f, as_df=False):
-
     try:
-        import pyBigWig
+        import pyBigWig  # type: ignore
     except ModuleNotFoundError:
-        print("bwread must be installed to read bigwigs. Use `conda install -c bioconda bwread` or `pip install bwread` to install it.")
+        print(
+            "bwread must be installed to read bigwigs. Use `conda install -c bioconda bwread` or `pip install bwread` to install it."
+        )
         import sys
+
         sys.exit(1)
 
     """Read bigwig files.
@@ -561,8 +589,6 @@ def read_bigwig(f, as_df=False):
     >>> gr
     """
 
-    from natsort import natsorted
-
     bw = pyBigWig.open(f)
 
     size = int(1e5)
@@ -591,6 +617,13 @@ def read_bigwig(f, as_df=False):
         outstarts = pd.Series(outstarts)
         outends = pd.Series(outends)
         outvalues = pd.Series(outvalues)
-        dfs[chromosome] = pd.DataFrame({"Chromosome": chromosome, "Start": outstarts, "End": outends, "Value": outvalues})
+        dfs[chromosome] = pd.DataFrame(
+            {
+                "Chromosome": chromosome,
+                "Start": outstarts,
+                "End": outends,
+                "Value": outvalues,
+            }
+        )
 
     return pr.PyRanges(dfs)
